#!/bin/sh
#
# Copyright (c) 2005  Peter Pentchev
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#
# $Ringlet: sh/misc/leafports/leafports.sh,v 1.6 2005/03/10 10:55:31 roam Exp $

usage()
{
	echo 'Usage: leafports [-dhLlVW]' 1>&2
	printf '\t-d\tdisplay debug information (LEAFDEBUG);\n' 1>&2
	printf '\t-h\tdisplay this help message and exit;\n' 1>&2
	printf '\t-L\tdo not display the leaf port names (NOLEAF);\n' 1>&2
	printf '\t-l\tdisplay port names only, not as diff output (ONLYLEAF);\n' 1>&2
	printf '\t-V\tdisplay program version information and exit;\n' 1>&2
	printf '\t-W\tdo not display the "weird" ports (NOWEIRD).\n' 1>&2
}

version()
{
	echo 'leafports 1.0pre2' 1>&2
	echo 'Copyright (c) 2005  Peter Pentchev' 1>&2
}

args=`getopt 'dhLlVW' "$@"`
if [ "$?" -ne 0 ]; then
	usage
	exit 1
fi
pleaseexit=''
set -- $args
for i; do
	case "$i" in
		-d)
			LEAFDEBUG=true
			shift;;
		-h)
			usage
			pleaseexit=true
			shift;;
		-l)
			ONLYLEAF=true
			shift;;
		-L)
			NOLEAF=true
			shift;;
		-V)
			version
			pleaseexit=true
			shift;;
		-W)
			NOWEIRD=true
			shift;;
		--)
			shift; break;;
	esac
done
if [ -n "$pleaseexit" ]; then
	exit 0
fi

TDIR=`mktemp -d -t leafports`
trap "rm -rf \"$TDIR\"/; echo 'leafports interrupted' 1>&2; exit 1" \
    TERM INT HUP

[ -n "${LEAFDEBUG}" ] && echo "Using temp dir $TDIR"

awk -F: '$1 == "@comment ORIGIN" { print $2 }' /var/db/pkg/*/+CONTENTS | \
	sort -u > "$TDIR"/origin.lst
awk -F: '$1 == "@comment DEPORIGIN" { print $2 }' /var/db/pkg/*/+CONTENTS | \
	sort -u > "$TDIR"/deporigin.lst
diff -u "$TDIR"/deporigin.lst "$TDIR"/origin.lst > "$TDIR"/origin.diff

if [ -z "${NOWEIRD}" ] && egrep -q '^-[^-]' "$TDIR"/origin.diff; then
	echo "WEIRD PORTS:"
	egrep '^-[^-]' "$TDIR"/origin.diff
	echo "END OF WEIRD PORTS"
fi

if [ -z "${NOLEAF}" ]; then
	if [ -z "${ONLYLEAF}" ]; then
		egrep '^\+[^+]' "$TDIR"/origin.diff
	else
		sed -ne '/^+[^+]/ { s/^+//; p; }' "$TDIR"/origin.diff
	fi
fi

rm -rf "$TDIR"/
